import 'package:chat_call_core/core/models/user_model.dart';
import 'package:dio/dio.dart';
import '../models/expert_model.dart';
import 'constaints.dart';
import 'package:injectable/injectable.dart';
import 'package:retrofit/http.dart';
import 'package:sharing/remote/model/base_list_response.dart';
import 'package:sharing/remote/model/base_object_response.dart';

part 'account_service.g.dart';

@injectable
@RestApi(baseUrl: ApiConstant.urlProduct)
abstract class AccountService {
  @factoryMethod
  factory AccountService(@Named('base_dio') Dio dio) = _AccountService;

  @POST("${ApiConstant.signIn}")
  Future<BaseObjectResponse<dynamic>> signIn(
      {@Field("device_id") required String deviceId,
      @Field("loginkey") required String loginKey,
      @Field("password") required String password});

  @POST(ApiConstant.signOut)
  Future<BaseObjectResponse<String>> signOut();

  @GET("${ApiConstant.experts}")
  Future<BaseListResponse<ExpertModel>> getExpertList(
      {@Query("specialize") required String? specialize});

  @GET("${ApiConstant.specializes}")
  Future<BaseListResponse<String>> getSpecializesList();

  @GET("${ApiConstant.profile}")
  Future<BaseObjectResponse<UserModel>> getUser();

  @POST(ApiConstant.devices)
  Future<BaseObjectResponse<UserModel>> devices(
      {@Field("device_id") required String? deviceId,
      @Field("device_type") required String? deviceType,
      @Field("device_name") required String? deviceName,
      @Field("os_version") required String? osVersion,
      @Field("app_version") required String? appVersion,
      @Field("device_token") required String? deviceToken});
}
