import 'package:injectable/injectable.dart';
import 'package:sharing/remote/network/either.dart';
import 'package:sharing/remote/network/fetch_data.dart';
import 'package:sharing/remote/network/fetch_failure.dart';
import 'package:sharing/remote/base_service.dart';

import '../../../../shared/helper/utils_helper.dart';
import '../../params/device_params.dart';
import '../account_repository.dart';
import '../account_service.dart';

@Injectable(as: AccountRepository)
class AccountRepositoryImpl extends BaseService with AccountRepository {
  final AccountService accountService;

  AccountRepositoryImpl(this.accountService);

  @override
  Future<void> signIn(
      {required String loginkey,
      required password,
      required deviceId,
      required Function(FetchData) funDataServer,
      required Function(FetchFailure) funError}) async {
    var deviceId = await Util.getDeviceId();
    final callApi = accountService.signIn(
      deviceId: deviceId,
      loginKey: loginkey,
      password: password,
    );
    await safeCallObjectStatus(
        response: await safeFetchV2(() => callApi),
        funDataServer: funDataServer,
        funError: funError);
  }

  @override
  Future<Either<FetchFailure, String?>> signOut() async {
    final callApi = accountService.signOut();
    return await safeCallMessageStatus(
        response: await safeFetchV2(() => callApi));
  }

  @override
  Future<void> getExpertList(
      {String? specialize,
      required Function(FetchData) funDataServer,
      required Function(FetchFailure) funError}) async {
    final callApi = accountService.getExpertList(specialize: specialize);
    await safeCallListStatus(
        response: await safeFetchV2(() => callApi),
        funDataServer: funDataServer,
        funError: funError);
  }

  @override
  Future<void> getSpecializesList(
      {required Function(FetchData) funDataServer,
      required Function(FetchFailure) funError}) async {
    final callApi = accountService.getSpecializesList();
    await safeCallListStatus(
        response: await safeFetchV2(() => callApi),
        funDataServer: funDataServer,
        funError: funError);
  }

  @override
  Future<void> getUser(
      {required Function(FetchData) funDataServer,
      required Function(FetchFailure) funError}) async {
    final callApi = accountService.getUser();
    await safeCallObjectStatus(
        response: await safeFetchV2(() => callApi),
        funDataServer: funDataServer,
        funError: funError);
  }

  @override
  Future<void> devices(
      {required DeviceParams params,
      required Function(FetchData) funDataServer,
      required Function(FetchFailure) funError}) async {
    final callApi = accountService.devices(
        deviceId: params.deviceId,
        deviceName: params.deviceName,
        deviceType: params.deviceType,
        deviceToken: params.deviceToken,
        appVersion: params.appVersion,
        osVersion: params.osVersion);
    await safeCallObjectStatus(
        response: await safeFetchV2(() => callApi),
        funDataServer: funDataServer,
        funError: funError);
  }
}
