import 'package:chat_call_core/core/local/shared_pref_source.dart';
import 'package:chat_call_core/core/modules/chat_types/src/user.dart';
import 'package:chat_call_core/presentation/chat/models/call_request_model.dart';
import 'package:chat_call_core/presentation/chat/models/chat_request_model.dart';
import 'package:chat_call_core/shared/base/base_page.dart';
import 'package:chat_call_core/shared/base/base_state.dart';
import 'package:chat_call_core/shared/di/injector.dart';
import 'package:chat_call_core/shared/helper/call_helper.dart';
import 'package:chat_call_core/shared/helper/safe_image.dart';
import 'package:chat_call_core/shared/helper/status_helper.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:flutter/material.dart';
import 'package:flutter_rating_bar/flutter_rating_bar.dart';
import 'package:flutter_svg/svg.dart';
import 'package:hainong_expert_module/shared/base/base_handle_mixin.dart';
import 'package:hainong_expert_module/shared/helper/date_helper.dart';
import 'package:hainong_expert_module/shared/helper/navigator_page_helper.dart';
import 'package:hainong_expert_module/shared/theme/colors.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import '../../../core/data/models/expert_model.dart';
import 'package:sharing/common/values/sizes.dart';
import 'package:sharing/widget/custom/spaces.dart';
import 'package:sharing/widget/text/custom_text_large_bold_field.dart';
import 'package:sharing/widget/text/custom_text_default_bold_field.dart';
import 'package:sharing/widget/text/custom_text_default_field.dart';
import '../../../shared/helper/permission_helper.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:sharing/widget/appbar/custom_app_bar_arrow.dart';
import 'expert_bloc.dart';
import 'package:sharing/common/values/hex_color.dart';
import 'package:sharing/widget/button/custom_button_label.dart';
import 'package:sharing/widget/pages/page_loading_ui_widget.dart';

class ExpertListPage extends BasePage {
  ExpertListPage({required this.callBackContact});

  final Function() callBackContact;

  @override
  _ExpertListPageState createState() => _ExpertListPageState(callBackContact);
}

class _ExpertListPageState extends BasePageState<ExpertBloc>
    with
        WidgetsBindingObserver,
        BaseHandleMixin,
        SingleTickerProviderStateMixin {
  List<ExpertModel>? data;
  String? domainUri = "";
  List<String> specializesData = [];
  List<Widget> specializesStatus = [];
  int selectStatusPosition = 1;
  String? specializesName = "";
  String? availableSupport = "agree";
  Function() callBackContact;
  GlobalKey _refresherKey = GlobalKey();
  RefreshController _refreshCtr = RefreshController(initialRefresh: false);

  _ExpertListPageState(this.callBackContact);

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget appTopBarMenu(BuildContext context) {
    return CustomAppBarArrow(
        backgroundColor: AppColors.primaryColor,
        title: "Chuyên gia",
        textColor: Colors.white,
        isShowShadow: false);
  }

  @override
  void callBackCalling(User user) {
    NavigatorPage.navigationCallPage(context,
        request: CallRequest(
          id: user.id.toString(),
          firebaseToken: "",
          image: user.imageUrl,
          name: user.firstName,
          isCalled: false,
          phone: user.phone,
          receiverId: user.id.toString(),
          dialerId: "",
          roomName: roomName,
          primaryColor: AppColors.colorBegin,
        ));
  }

  @override
  void initBloc() {
    bloc?.add(GetSpecializesistEvent());
    Future.delayed(Duration(seconds: 1), () {
      bloc?.add(GetExpertListEvent());
    });
    Future.delayed(Duration(seconds: 2), () {
      bloc?.add(GetUserEvent());
    });
    scaffoldBackgroundColor = AppColors.primaryColor;
    PermissionHelper()
        .requestPermissions([Permission.camera, Permission.microphone]);
    WidgetsBinding.instance?.addObserver(this);
  }

  @override
  void refreshPage() {
    bloc?.add(GetUserEvent());
    bloc?.add(GetExpertListEvent(specialize: specializesName));
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void listener(BuildContext context, BaseState state) {
    if (state is SuccessDataState<List<ExpertModel>>) {
      setState(() => data = state.data);
    } else if (state is SuccessDataState<List<String>>) {
      setState(() {
        if (state.data != null) {
          specializesData = state.data!;
        }
        specializesData.insert(0, "Tất cả");
      });
    }
  }

  @override
  Widget builder(BuildContext context, BaseState state) {
    return Scaffold(
        backgroundColor: Colors.white,
        body: Column(
          children: [
            SpaceH2(),
            specializesWidget(),
            Row(children: [
              Expanded(
                child: expertStatusItem(
                    item: StatusSpecializes(
                        id: 1, title: "Hỗ trợ", content: "Thảo luận bây giờ")),
              ),
              Expanded(
                child: expertStatusItem(
                    item: StatusSpecializes(
                        id: 2, title: "Đang bận", content: "Để lại lời nhắn")),
              ),
              Expanded(
                child: expertStatusItem(
                    item: StatusSpecializes(
                        id: 3, title: "Tất cả", content: "Chuyên gia")),
              )
            ]),
            data != null
                ? Expanded(
                    child: SmartRefresher(
                        physics: BouncingScrollPhysics(),
                        header: WaterDropMaterialHeader(
                          color: AppColors.primaryColor,
                          backgroundColor: Colors.white,
                        ),
                        enablePullDown: true,
                        key: _refresherKey,
                        controller: _refreshCtr,
                        onRefresh: () async {
                          isLoading = true;
                          await Future.delayed(Duration(milliseconds: 200), () {
                            initBloc();
                            _refreshCtr.refreshCompleted();
                          });
                        },
                        child: ListView.builder(
                          shrinkWrap: true,
                          physics: NeverScrollableScrollPhysics(),
                          padding:
                              EdgeInsets.symmetric(vertical: Sizes.PADDING_12),
                          scrollDirection: Axis.vertical,
                          itemBuilder: (context, index) =>
                              itemExpertBuilder(context, data![index], index),
                          itemCount: data?.length ?? 0,
                        )))
                : Container(
                    padding: EdgeInsets.symmetric(vertical: Sizes.PADDING_20),
                    child: LoadingUIPageWidget())
          ],
        ));
  }

  bool isShowExprert(String? isSuport) {
    if (isSuport != null) {
      if (availableSupport == "all") {
        return true;
      } else {
        if (availableSupport == isSuport) {
          return true;
        } else {
          return false;
        }
      }
    } else {
      if (availableSupport == "all") {
        return true;
      } else {
        if (availableSupport == "agree") {
          return false;
        } else if (availableSupport == "disagree") {
          return true;
        } else {
          return true;
        }
      }
    }
  }

  bool isHiddenExpert(String? isSuport) {
    if (isSuport != null) {
      if (isSuport == "agree") {
        return true;
      } else {
        return false;
      }
    } else {
      return false;
    }
  }

  Widget itemExpertBuilder(BuildContext context, ExpertModel? data, int index) {
    final isAvailable = isShowExprert(data?.availableSuport);
    final isHidden = isHiddenExpert(data?.availableSuport);
    if (isAvailable) {
      return GestureDetector(
        onTap: () {},
        child: Container(
          padding: EdgeInsets.symmetric(
              vertical: Sizes.PADDING_12, horizontal: Sizes.PADDING_12),
          margin: EdgeInsets.symmetric(
              horizontal: Sizes.PADDING_10, vertical: Sizes.PADDING_10),
          decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.circular(Sizes.RADIUS_16),
            boxShadow: [
              BoxShadow(
                color: Colors.grey.withOpacity(0.3),
                spreadRadius: 4,
                blurRadius: 5,
                offset: Offset(0, 3),
              ),
            ],
          ),
          child: Column(
            children: [
              Row(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Stack(
                    children: [
                      Container(
                        decoration: BoxDecoration(
                            border: Border.all(color: Colors.green, width: 1),
                            shape: BoxShape.circle),
                        child: SafeImage(
                          domainUri: data?.image?.contains("hainong.vn") == true
                              ? ""
                              : domainUri,
                          imageUrl: data?.image,
                          package: "hainong_expert_module",
                          radius: 120,
                          width: Sizes.WIDTH_80,
                          height: Sizes.WIDTH_80,
                        ),
                      ),
                      Positioned(
                        bottom: 4,
                        right: 2,
                        child: Container(
                          width: Sizes.WIDTH_16,
                          height: Sizes.WIDTH_16,
                          decoration: BoxDecoration(
                              shape: BoxShape.circle,
                              color: data?.currentDeviceToken != null
                                  ? Colors.green
                                  : Colors.yellow),
                        ),
                      )
                    ],
                  ),
                  SpaceW12(),
                  Padding(
                    padding: EdgeInsets.symmetric(vertical: Sizes.PADDING_4),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        CustomTextLargeBoldField(
                          label: "${data?.name}",
                          textSize: Sizes.TEXT_SIZE_18,
                          maxLine: 1,
                        ),
                        SpaceH6(),
                        RatingBar.builder(
                          initialRating: 3,
                          tapOnlyMode: false,
                          direction: Axis.horizontal,
                          itemSize: 16,
                          allowHalfRating: false,
                          itemCount: 5,
                          itemBuilder: (context, _) => Icon(
                            Icons.star,
                            color: Colors.amber,
                          ),
                          onRatingUpdate: (double value) {},
                        ),
                        SpaceH6(),
                        CustomTextDefaultField(
                          label:
                              "Hỗ trợ: ${DateHelper.dateFormatDatetime(time: data?.startWorkingTime, content: "00 : 01 Sáng")} - ${DateHelper.dateFormatDatetime(time: data?.endWorkingTime, content: "11 : 59 Chiều")}",
                        )
                      ],
                    ),
                  ),
                ],
              ),
              SpaceH12(),
              Padding(
                padding: EdgeInsets.symmetric(horizontal: Sizes.PADDING_12),
                child: Column(
                  children: [
                    Row(
                      children: [
                        CustomTextDefaultField(
                            label: "Nhóm hỗ trợ: ", color: HexColor("#505050")),
                        SpaceW8(),
                        CustomTextDefaultField(
                            label: data?.specialize ?? "Chưa xác định",
                            textSize: Sizes.TEXT_SIZE_16,
                            color: Colors.black87),
                      ],
                    ),
                    SpaceH6(),
                    Row(
                      children: [
                        CustomTextDefaultField(
                            label: "Kinh nghiệm: ", color: HexColor("#505050")),
                        SpaceW8(),
                        CustomTextDefaultField(
                            label: data?.memberRate ?? "Chưa xác định",
                            textSize: Sizes.TEXT_SIZE_16,
                            color: Colors.black87),
                      ],
                    ),
                  ],
                ),
              ),
              SpaceH12(),
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: [
                  Container(
                    height: Sizes.HEIGHT_36,
                    width: Sizes.WIDTH_170,
                    child: CustomButtonLable(
                        width: Sizes.WIDTH_160,
                        borderRadius: Sizes.RADIUS_32,
                        icon: Image.asset(
                          "assets/images/ic_support.png",
                          width: Sizes.WIDTH_24,
                          height: Sizes.WIDTH_24,
                          color: Colors.white,
                          package: "hainong_expert_module",
                        ),
                        label: "Gọi chuyên gia",
                        color: isHidden ? HexColor("#3E97F0") : Colors.grey,
                        onPress: () {
                          if (isHidden == true) {
                            navigatorChatRoom(data);
                          } else {
                            customShowDialogWaring(
                                msg: ["Chuyên gia đang bận"]);
                          }
                        }),
                  ),
                  Container(
                    height: Sizes.HEIGHT_36,
                    width: Sizes.WIDTH_120,
                    child: CustomButtonLable(
                        borderRadius: Sizes.RADIUS_32,
                        width: Sizes.WIDTH_140,
                        icon: Padding(
                          padding:
                              EdgeInsets.symmetric(horizontal: Sizes.PADDING_4),
                          child: SvgPicture.asset(
                            "assets/images/ic_info.svg",
                            package: "hainong_expert_module",
                          ),
                        ),
                        label: "Hỏi đáp",
                        color: HexColor("#DC1F58"),
                        onPress: () {
                          callBackContact();
                        }),
                  )
                ],
              )
            ],
          ),
        ),
      );
    } else {
      return SizedBox();
    }
  }

  Widget specializesWidget() {
    if (specializesData.isNotEmpty == true) {
      return Container(
          padding: EdgeInsets.symmetric(horizontal: Sizes.PADDING_2),
          child: DropdownSearch<String>(
              mode: Mode.MENU,
              items: specializesData,
              selectedItem: specializesData.first,
              onChanged: (data) {
                if (data != null) {
                  specializesName = data;
                  bloc?.add(GetExpertListEvent(
                      specialize: specializesName == "Tất cả"
                          ? null
                          : specializesName));
                }
              },
              dropdownSearchDecoration: InputDecoration(
                border: OutlineInputBorder(
                    borderSide: BorderSide(color: Colors.grey, width: 1)),
                contentPadding: EdgeInsets.all(Sizes.PADDING_16),
                errorStyle:
                    TextStyle(color: Colors.red, fontSize: Sizes.TEXT_SIZE_16),
              )));
    } else {
      return Container();
    }
  }

  Widget expertStatusItem({required StatusSpecializes item}) {
    return GestureDetector(
      onTap: () {
        setState(() {
          selectStatusPosition = item.id ?? 1;
          switch (item.id) {
            case 1:
              availableSupport = "agree";
              break;
            case 2:
              availableSupport = "disagree";
              break;
            default:
              availableSupport = "all";
          }
          bloc?.add(GetExpertListEvent(
              specialize:
                  specializesName == "Tất cả" ? null : specializesName));
        });
      },
      child: Container(
          margin: EdgeInsets.symmetric(
              horizontal: Sizes.PADDING_2, vertical: Sizes.PADDING_2),
          padding: EdgeInsets.symmetric(
              horizontal: Sizes.PADDING_12, vertical: Sizes.PADDING_12),
          decoration:
              BoxDecoration(border: Border.all(color: Colors.grey, width: 1)),
          child: Column(children: [
            CustomTextDefaultBoldField(
              label: item.title,
              color: item.id == selectStatusPosition
                  ? Colors.black87
                  : HexColor("#828282"),
              maxLine: 1,
              textSize: Sizes.TEXT_SIZE_16,
            ),
            SpaceH6(),
            CustomTextDefaultField(
              label: item.content,
              color: item.id == selectStatusPosition
                  ? Colors.black87
                  : HexColor("#828282"),
              maxLine: 1,
              textSize: Sizes.TEXT_SIZE_12,
            ),
          ])),
    );
  }

  void navigatorChatRoom(ExpertModel? data) {
    if (data != null) {
      if (data.currentDeviceToken != null) {
        final primaryColor = getIt<SharePrefSource>().getPrimaryColor();
        NavigatorPage.navigatorChatPage(
          context,
          request: ChatRequest(
              id: data.id.toString(),
              userId: data.id.toString(),
              firebaseToken: data.currentDeviceToken,
              callType: "chat",
              email: data.email,
              image: data.image,
              name: data.name,
              phone: data.phone,
              roomName: "${currentUser?.id.toString()}-${data.id.toString()}",
              isExpert: false,
              themePrimaryColor: primaryColor),
        );
      } else {
        customShowDialogWaring(
            msg: [StatusHelper.getMessageCallError(errorCode: 400)]);
      }
    } else {
      customShowDialogWaring(
          msg: [StatusHelper.getMessageCallError(errorCode: 502)]);
    }
  }
}

class StatusSpecializes {
  final int? id;
  final String? title;
  final String? content;
  StatusSpecializes({this.id, this.title, this.content});
}
