import 'package:chat_call_core/presentation/chat/models/chat_request_model.dart';
import 'package:chat_call_core/shared/base/base_page.dart';
import 'package:chat_call_core/shared/base/base_state.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:fcm_config/fcm_config.dart';
import 'package:flutter/material.dart';
import 'package:flutter_animated_dialog/flutter_animated_dialog.dart';
import 'package:hainong_expert_module/presentation/pages/expert/expert_bloc.dart';
import 'package:sharing/common/values/sizes.dart';
import 'package:sharing/widget/appbar/custom_app_bar_arrow.dart';
import 'package:sharing/widget/button/custom_button_label.dart';
import 'package:sharing/widget/custom/spaces.dart';
import 'package:sharing/widget/text/custom_text_default_field.dart';
import 'package:sharing/widget/text/custom_text_medium_bold_field.dart';
import 'package:sharing/widget/text/custom_text_medium_field.dart';
import 'package:chat_call_core/shared/helper/safe_image.dart';
import 'package:sharing/common/values/hex_color.dart';

class ProfilePage extends BasePage {
  ProfilePage({required this.data});

  final ChatRequest? data;

  @override
  _AccountPageState createState() => _AccountPageState(data);
}

class _AccountPageState extends BasePageState<ExpertBloc> {
  final ChatRequest? data;

  _AccountPageState(this.data);

  @override
  Widget appTopBarMenu(BuildContext context) {
    return CustomAppBarArrow(
      title: "Thông tin người dùng",
      textColor: Colors.white,
      backgroundColor: HexColor(data?.themePrimaryColor),
    );
  }

  @override
  void initBloc() {
    // scaffoldBackgroundColor = HexColor(data?.themePrimaryColor);
  }

  @override
  void listener(BuildContext context, BaseState state) {}

  @override
  Widget builder(BuildContext context, BaseState state) {
    if (data != null) {
      return Container(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [
            Container(
              color: HexColor(data?.themePrimaryColor),
              child: Column(
                children: [
                  SpaceH20(),
                  SafeImage(
                      imageUrl: "${data?.image}",
                      radius: 160,
                      width: Sizes.WIDTH_100,
                      height: Sizes.WIDTH_100,
                      package: "hainong_expert_module"),
                  SpaceH12(),
                  CustomTextMediumField(
                      color: Colors.white,
                      label: data?.name,
                      textSize: Sizes.TEXT_SIZE_18),
                  SpaceH6(),
                  CustomTextMediumField(
                      color: Colors.white,
                      label: data?.phone,
                      textSize: Sizes.TEXT_SIZE_20),
                  SpaceH6(),
                  if (data?.email?.isNotEmpty == true) ...[
                    CustomTextMediumField(
                        color: Colors.white,
                        label: data?.email ?? "",
                        textSize: Sizes.TEXT_SIZE_20),
                  ],
                  SpaceH20()
                ],
              ),
            ),
          ],
        ),
      );
    } else {
      return Container(
          alignment: Alignment.center,
          child: CustomTextDefaultField(
              align: TextAlign.center,
              textSize: Sizes.TEXT_SIZE_18,
              label:
                  "Hiện không có thông tin người dùng \n Hãy đăng nhập để biết thêm thông tin"));
    }
  }

  Future<void> customShowDialogExit() async {
    await showAnimatedDialog(
      context: context,
      barrierDismissible: true,
      builder: (BuildContext context) {
        return Dialog(
          alignment: Alignment.center,
          shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.all(Radius.circular(12))),
          child: Container(
            height: Sizes.HEIGHT_160,
            padding: EdgeInsets.only(
                left: Sizes.PADDING_16,
                right: Sizes.PADDING_16,
                top: Sizes.PADDING_16),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                CustomTextMediumBoldField(
                    align: TextAlign.center,
                    label: "Bạn có chắc muốn đăng xuất \n khỏi ứng dụng ?",
                    maxLine: 2),
                SpaceH20(),
                Container(
                  color: Colors.white,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Expanded(
                          child: CustomButtonLable(
                              label: "No".tr(),
                              color: Colors.grey,
                              onPress: () {
                                Navigator.of(context).pop();
                              })),
                      SpaceW12(),
                      Expanded(
                          child: CustomButtonLable(
                              label: "Đồng ý".tr(),
                              onPress: () async {
                                await FirebaseMessaging.instance.deleteToken();
                                bloc?.add(SignOutEvent());
                              })),
                    ],
                  ),
                )
              ],
            ),
          ),
        );
      },
      animationType: DialogTransitionType.fadeScale,
      curve: Curves.fastOutSlowIn,
      duration: const Duration(milliseconds: 100),
    );
  }
}
