import 'dart:async';
import 'package:chat_call_core/core/local/shared_pref_source.dart';
import 'package:chat_call_core/core/models/user_model.dart';
import 'package:chat_call_core/shared/di/injector.dart';
import 'package:chat_call_core/shared/helper/dialog_helper.dart';
import 'package:flutter/material.dart';
import 'package:sharing/common/utils/enum/supported_locale.dart';
import '../../presentation/pages/expert_page.dart';
import '../../presentation/app/app_state.dart';

mixin BaseHandleMixin<T extends StatefulWidget> on State<T> {
  bool? isNumberBadge = false;
  bool? isLoading = false;
  int? numberBadge = 0;
  int? invoiceId;
  UserModel? currentUser;
  SupportedLocale? locale = SupportedLocale.vi;
  StreamSubscription? appSubcription;

  @override
  void initState() {
    initData();
    registerAppBloc();
    super.initState();
  }

  @override
  void dispose() {
    appSubcription?.cancel();
    super.dispose();
  }

  void initData() {
    currentUser = getIt<SharePrefSource>().getUser();
  }

  void registerAppBloc() {
    appSubcription?.cancel();
    appSubcription = appBloc.stream.listen((state) {
      if (state is ShowDialogState) {
        DialogHelper.customShowSnackbar(context, status: state.status);
      }
    });
  }
}
