import 'dart:developer';

import 'package:chat_call_core/call_core.dart';
import 'package:chat_call_core/core/local/shared_pref_source.dart';
import 'package:chat_call_core/presentation/call/call_bloc.dart';
import 'package:chat_call_core/presentation/call/call_page.dart';
import 'package:chat_call_core/presentation/call/call_suggest_page.dart';
import 'package:chat_call_core/presentation/chat/local/chat_page.dart';
import 'package:chat_call_core/presentation/chat/models/call_data_model.dart';
import 'package:chat_call_core/presentation/chat/models/call_request_model.dart';
import 'package:chat_call_core/presentation/chat/models/call_response_model.dart';
import 'package:chat_call_core/presentation/chat/models/chat_request_model.dart';
import 'package:chat_call_core/shared/di/injector.dart';
import 'package:chat_call_core/shared/helper/dialog_helper.dart';
import 'package:chat_call_core/shared/helper/status_helper.dart';
import 'package:flutter/material.dart';
import 'package:hainong_expert_module/presentation/pages/profile/profile_page.dart';

import '../../presentation/pages/expert_page.dart';
import '../theme/colors.dart';

class NavigatorPage {
  static void navigatorChatPage(BuildContext context,
      {required ChatRequest request, String status = ""}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) {
      final primaryColor = getIt<SharePrefSource>().getPrimaryColor();
      return ChatPage(
        chatRequest: request,
        callBackAudioCall: () {
          if (request.userId.isNotEmpty == true) {
            NavigatorPage.navigationCallPage(context,
                request: CallRequest(
                    isExpert: false,
                    id: request.id,
                    firebaseToken: "",
                    name: request.name,
                    email: request.email,
                    image: request.image,
                    callChoose: CallChoose.audioCall,
                    isCalled: request.isCalled,
                    phone: request.phone,
                    receiverId: request.userId.toString(),
                    dialerId: "",
                    roomName: request.roomName,
                    primaryColor: primaryColor));
          } else {
            DialogHelper.customShowSnackbar(context,
                title: "Thông báo",
                message: StatusHelper.getMessageCallError(errorCode: 401));
          }
        },
        callBackVideoCall: () {
          if (request.userId.isNotEmpty == true) {
            NavigatorPage.navigationCallPage(context,
                request: CallRequest(
                    isExpert: false,
                    id: request.id,
                    firebaseToken: "",
                    name: request.name,
                    email: request.email,
                    image: request.image,
                    callChoose: CallChoose.videoCall,
                    isCalled: request.isCalled,
                    phone: request.phone,
                    receiverId: request.userId.toString(),
                    dialerId: "",
                    roomName: request.roomName,
                    primaryColor: primaryColor));
          } else {
            DialogHelper.customShowSnackbar(context,
                message: StatusHelper.getMessageCallError(errorCode: 401));
          }
        },
        callBackCall: (data) {
          NavigatorPage.navigationCallPage(context,
              request: CallRequest(
                  id: data.id,
                  firebaseToken: "",
                  callChoose: CallChoose.videoCall,
                  name: request.name,
                  email: request.email,
                  image: request.image,
                  isCalled: false,
                  phone: request.phone,
                  receiverId: request.userId.toString(),
                  dialerId: request.dialerId,
                  roomName: request.roomName,
                  primaryColor: AppColors.colorBegin));
        },
        callBackProfile: () {
          Navigator.push(context, MaterialPageRoute(builder: (context) {
            return ProfilePage(data: request);
          }));
        },
      );
    }));
  }

  static void navigationCallSuggestPage(BuildContext context,
      {required CallResponse result, required Function() callBackAccept}) {
    CallCore.setShowBarCall(true);
    var request = ChatRequest(
        id: result.id.toString(),
        userId: result.receiverId!,
        roomName: result.roomName,
        image: result.image,
        name: result.name,
        firebaseToken: result.dialerDeviceToken,
        callType: result.callType,
        isCalled: false,
        isChatInVideo: false,
        isAutoCall: true);
    Navigator.push(context, MaterialPageRoute(builder: (context) {
      return CallSuggestPage(
          request: request,
          callBackCancel: () => Navigator.of(context).pop(false),
          callBackAccept: () => Navigator.of(context).pop(true));
    })).then((value) {
      if (value == true) {
        callBackAccept();
      } else if (value == false) {
        sendCallCancel(
            id: request.id,
            roomName: result.roomName!,
            tokenFirebase: result.dialerDeviceToken!);
      } else {
        appCallBloc.add(ShowCallbarEvent(true));
      }
    });
  }

  static void navigationCallPage(BuildContext context,
      {required CallRequest request, Function()? callBack}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) {
      return CallPage(request: request);
    })).then((result) {
      try {
        final data = result as CallData?;
        appCallBloc.add(RefreshPageEvent());
        DialogHelper.customShowSnackbar(context,
            message: data?.message, status: data?.status, callBack: () {
          DialogHelper.showBottomSheet(context,
              id: request.receiverId.toString(),
              colorPrimary: request.primaryColor ?? AppColors.colorBegin);
        });
      } catch (e) {
        log(e.toString());
      }
      if (callBack != null) callBack();
    });
  }

  static void navigatorCallPageTranfer(BuildContext context,
      {required CallResponse result, String? privateColor}) {
    var request = CallRequest(
      id: result.id.toString(),
      firebaseToken: result.dialerDeviceToken,
      image: result.image,
      isCalled: false,
      phone: result.phone,
      callChoose: convertToEnumCallChoose(result.callType ?? "video"),
      isExpert: true,
      name: result.name,
      receiverId: result.receiverId.toString(),
      dialerId: result.dialerId.toString(),
      roomName: result.roomName,
      primaryColor: privateColor,
    );
    navigationCallPage(context, request: request);
  }

  static snackBar(BuildContext context, String? message) {
    return ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        content: Text(message!),
        duration: const Duration(seconds: 4),
      ),
    );
  }

  static void sendCallCancel(
      {required String id,
      required String roomName,
      required String tokenFirebase}) {
    final status = "actionCallDecline";
    appBloc.add(UpdateCallStatusEvent(
        callType: "call",
        id: id,
        status: status,
        message: StatusHelper.getMessageCall(status: status)));
  }
}
